/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package graphicChat.model;


import graphicChat.logger.OhmLogger;

import java.io.BufferedReader;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Flow;
import java.util.concurrent.SubmissionPublisher;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author hd, chris
 */
public class ChatModel implements Runnable
{
  private static final Logger lg = OhmLogger.getLogger();
  
  private final ExecutorService eService;
  private final SubmissionPublisher<List<Figure>> iPublisher;  
  private volatile boolean laufend;
  private final GraphicData gdata;
  
  List<Figure> msg;
  
//  BufferedReader in;
//  PrintWriter out;
  
  ObjectOutputStream objectOutput;
  ObjectInputStream objectInput;        
  
  private Socket socket;

  
  public ChatModel()
  {
    laufend = false;
    iPublisher = new SubmissionPublisher<>();
    eService = Executors.newSingleThreadExecutor();    
    gdata = new GraphicData();
  }

  public void addSubscription(Flow.Subscriber<List<Figure>> subscriber)
  {
    iPublisher.subscribe(subscriber);
  }
  
  public synchronized void start(){
    laufend = true;
    eService.submit(this);
    this.notifyAll();//muss sync
    lg.info("startet");
  }

  @Override
  public void run()
  { 
    while(true){
      try {
        try
        {
         msg = (List<Figure>)objectInput.readObject();
         lg.info("Nachricht empfangen: " + String.valueOf(msg.size()));
        }
        catch (ClassNotFoundException ex)
        {
          lg.log(Level.SEVERE, null, ex);
        }
        addList(msg);
        iPublisher.submit(msg);//wenn neue Nachricht
      } catch (IOException ex) {
        lg.log(Level.SEVERE, ex.toString());
      }
    } 
  }

  public void setSocket(Socket s) throws IOException 
  {
    InputStream iStream = s.getInputStream();
    OutputStream oStream = s.getOutputStream();
    
    objectOutput = new ObjectOutputStream(oStream);
//    objectOutput.writeObject(msg);  

    objectInput = new ObjectInputStream(iStream);
    start();
  }

  public void setClient(int PORT, String IP_ADRESSE) throws IOException
  {
    Thread connectThread = new Thread(new Runnable() {
      @Override
      public void run() {
        try {
          lg.info("Client: verbinde ...");
          Socket s = new Socket(IP_ADRESSE, PORT); // Achtung: blockiert!
          setSocket(s);
          lg.info("Client: Verbindung hergestellt");
        } catch (Exception e) {
//            e.printStackTrace();
            lg.info("Client: Verbindung fehlgeschlagen: " + e);
        }
      }
    });
    connectThread.start();
  }
  
  public void setServer(int PORT) throws IOException
  {
    Thread connectThread;
    connectThread = new Thread(new Runnable() {
      @Override
      public void run() {
        try {
          ServerSocket sSocket = new ServerSocket(PORT);
          lg.info("Server: Warte auf Verbindung ...");
          Socket s = sSocket.accept(); // Achtung: blockiert!
          setSocket(s);
          lg.info("Server: Verbindung akzeptiert");
        } catch (IOException e) {
//          e.printStackTrace();
          lg.info("Server: Verbindung fehlgeschlagen: " + e);
        }
      }
    });
    connectThread.start();
  }

  public void sendMessage(List<Figure> msg)
  {
    if(laufend){
      lg.log(Level.INFO, "Sende Nachricht: " + String.valueOf(msg.size()));
      try  
      {
        List<Figure> cloned_list = new ArrayList<Figure>(msg); 
        objectOutput.writeObject(cloned_list);
        objectOutput.flush();
      }
      catch (IOException ex)
      {
          Logger.getLogger(ChatModel.class.getName()).log(Level.SEVERE, null, ex);
      }
    }
    else{
      lg.log(Level.INFO, "Keine Verbindung!");
    }
  }
  
  public Figure addFigure()
  {
    return gdata.addFigure();
  }

  public List<Figure> getFigures()
  {
    return gdata.getFigures();
  }
  public List<Figure> getSendFigures()
  {
    return gdata.getSendFigures();
  }
  
  public void addList(List<Figure> list)
  {
    gdata.addList(list);  
  }
  public void saveFigures(List<Figure> list)
  {
    gdata.saveFigures(list);  
  }
  
}